/**
* Copyright (c) Pervasive Displays Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
**/

#ifndef PDI_DISPLAYS_DRIVERS_FPL_DRIVERS_ITC_ENGINE_H_
#define PDI_DISPLAYS_DRIVERS_FPL_DRIVERS_ITC_ENGINE_H_
#include <stdint.h>
#include <string.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>
#include <math.h>
#include "EPD_drivers.h"

#define TP_Empty			(uint8_t)(-128)
#define sig_TP_Empty		(int8_t)(-128)
typedef void (*ImageData_handler)(uint8_t *LineData, uint16_t len, uint8_t PixelBits);

#define Res_CR_OK		0x01
#define Res_TP_OK		0x02
typedef enum __attribute__((packed))
{
    CM_Monochrome,
    CM_Color,
    CM_ALL
}
ColorMode_t;

typedef enum __attribute__((packed))
{
    io_All,
    io_Panel_on,
    io_Res,
    io_CS,
    io_DC,
    io_discharge,
    io_SPI_Pin
}
IO_Define_t;

// iCMD means commands for iTC, Cr = commands for Spectra, TP = commands for temperature
// IO_Ctrl = commands for ontrolling GPIO of IO_Define_t, image = commands for sending image data
typedef enum __attribute__((packed))
{
    iCMD_END,
    iCMD_iTC_SPI,
    iCMD_iTC_SPI_Cr,
    iCMD_iTC_SPI_TP,
    iCMD_iTC_SPI_Cr_TP,
    iCMD_Busy = 0x10,
    iCMD_IO_Ctrl,
    iCMD_Delay,
    iCMD_Image,
    iCMD_Image_PU,
    iCMD_SetTemper
}
iTC_Cmd_t;

typedef struct __attribute__((__packed__))
{
    uint16_t 		time;
}
iTC_delay_param_t;

typedef struct __attribute__((__packed__))
{
    uint8_t 		io_pin;
    uint8_t 		io_data;
}
iTC_IO_param_t;

typedef struct __attribute__((__packed__))
{
    uint8_t 		compare_data;
    uint16_t 		time;
}
iTC_Busy_param_t;
#define RegTemper_Header		2
typedef struct __attribute__((__packed__))
{
    uint8_t 		regIndex_cmd;
    uint8_t 		DataLen;
    uint8_t 		*reg_data;
}
iTC_Reg_Temper_t;
#define RegData_Header		2
typedef struct __attribute__((__packed__))
{
    uint8_t 		regIndex_cmd;
    uint8_t 		DataLen;
    uint8_t 		*reg_data;
}
iTC_Reg_data_t;

#define RegCr_Header		3
typedef struct __attribute__((__packed__))
{
    ColorMode_t  	CrMode;
    uint8_t 		regIndex_cmd;
    uint8_t 		DataLen;
    uint8_t 		*reg_data;
}
iTC_Reg_Cr_t;

#define RegCrTP_Header		5
typedef struct __attribute__((__packed__))
{
    ColorMode_t  	CrMode;
    int8_t 			Tmax;
    int8_t 			Tmin;
    uint8_t 		regIndex_cmd;
    uint8_t 		DataLen;
    uint8_t 		*reg_data;
}
iTC_Reg_Cr_TP_t;

#define RegTP_Header		4
typedef struct __attribute__((__packed__))
{
    int8_t 			Tmax;
    int8_t 			Tmin;
    uint8_t 		regIndex_cmd;
    uint8_t 		DataLen;
    uint8_t 		*reg_data;
}
iTC_Reg_TP_t;


#define RegImage_Header		9
typedef struct __attribute__((__packed__))
{
    ColorMode_t  	CrMode;
    bool 			UseInternalData;
    uint8_t 		InternalData;
    uint8_t 		pixel_bit;
    uint8_t 		regIndex_cmd;
    uint16_t 		Horizontal;
    int16_t 		Vertical;
    //uint8_t 		PixelBits;
}
iTC_Reg_Image_t;

typedef struct __attribute__((__packed__))
{
    const uint8_t 		*PowerOn_LUT;
    const uint8_t 		*Initial_LUT;
    const uint8_t			*Update_LUT;
    const uint8_t			*PowerOffLUT;
    const uint8_t			*ExceptionLUT;
}
iTC_WaveformLUT_t;


class iTC_EngineClass : public Board_HW_DriverClass
{
private:
    void readLUTData(uint8_t *outputData, uint8_t len);
    uint8_t decod_cmd(void);

public:

    void iTC_Engine_init(ColorMode_t cr, int8_t temper, ImageData_handler OnImageData_handler, EPD_read_memory_handler OnReadExternalMemory);
    uint8_t RunEngine(const void *runLUTFile);
    uint8_t Ex_RunEngine(long ExtenalAddress);
    void SetPU_ImageDataHandle(ImageData_handler OnImageDataPU_handler);

    void iTC_IO_Config(void);
    void iTC_power_on(uint8_t EPD_type_index, int8_t temperature, OperationMode_t operation_mode, const iTC_WaveformLUT_t *GU_LUT,
                      const iTC_WaveformLUT_t *OTP_LUT, const iTC_WaveformLUT_t *PU_LUT);
    uint8_t iTC_initialize_driver(OperationMode_t operation_mode);
    uint8_t iTC_Display_from_Ram(const uint8_t *previous_image_ptr, const uint8_t *new_image_ptr);
    uint8_t iTC_Display_from_flash(long previous_image_flash_address, long new_image_flash_address, EPD_read_memory_handler On_EPD_read_flash);
    uint8_t iTC_power_off();

    void iTC_Display_PU_from_Ram(uint8_t *previous_image_ptr, uint8_t *new_image_ptr, uint16_t PU_Time);
    void iTC_Display_PU_From_Flash(long previous_image_flash_address, long new_image_flash_address, EPD_read_memory_handler On_EPD_read_flash, uint16_t PU_Time);


};


#endif /* PDI_DISPLAYS_DRIVERS_FPL_DRIVERS_ITC_ENGINE_H_ */
